'' =================================================================================================
''
''   File....... jm_ds3231_demo.spin
''   Purpose.... DS3231 demo
''   Author..... Jon "JonnyMac" McPhalen  
''               Copyright (C) 2016 Jon McPhalen    
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 19 DEC 2016
''
'' =================================================================================================


con { timing }

  _clkmode = xtal1 + pll16x                                     
  _xinfreq = 5_000_000                                          ' use 5MHz crystal

  CLK_FREQ = (_clkmode >> 6) * _xinfreq                         ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us


con { io pins }

  RX1  = 31                                                     ' programming / terminal
  TX1  = 30

  SDA1 = 29                                                     ' eeprom / i2c
  SCL1 = 28


con

  SDA2 =  5
  SCL2 =  4


obj

' main                                                          ' * master Spin cog
  time : "jm_time"                                              '   timing and delays
  io   : "jm_io_basic"                                          '   essential io
  rtc  : "jm_ds3231"                                            '   rtc
  term : "jm_fullduplexserial"                                  ' * serial IO for terminal
                                                                 
' * uses cog when loaded                                         
                                                                 
                                                                 
var

  byte  clock[7]                                                           
                                                                 
                                                                 
pub main | last                                                        
                                                                 
  setup

  set_clock_regs

  last := -1

  term.tx(term#CLS)

  repeat
    rtc.read(7, @clock, rtc#R_SCS)
    if (clock[0] <> last)
      refresh_display
      last := clock[rtc#R_SCS]
                                                               
                                                                 
pub setup                                                        
                                                                 
'' Setup IO and objects for application                          
                                                                 
  time.start                                                    ' setup timing & delays

  io.start(0, 0)                                                ' clear all pins (master cog)

  rtc.start(SCL2, SDA2)                                         ' connect to DS3231
                                                                 
  term.start(RX1, TX1, %0000, 115_200)                          ' start serial for terminal


pub refresh_display | idx, tmp

  term.tx(term#HOME)

  repeat idx from 0 to 6
    term.hex(clock[idx], 2)
    term.tx(" ")

  repeat 2
    term.tx(13)  

  term.str(rtc.day_short(clock[rtc#R_DAY]))
  term.tx(term#CLREOL)
  term.tx(13)

  term.str(rtc.time_hhmmss(@clock[rtc#R_SCS])) 
  term.tx(13)

  term.str(rtc.time_ampm(@clock[rtc#R_SCS]))
  repeat 2
    term.tx(13)

  tmp := rtc.temp_c
  term.rjdec(tmp / 100, 3, " ")
  term.tx(".")
  term.rjdec(tmp // 100, 2, "0")
  term.tx("C")
  term.tx(13)

  tmp := rtc.temp_f
  term.rjdec(tmp / 100, 3, " ")
  term.tx(".")
  term.rjdec(tmp // 100, 2, "0")
  term.tx("F")
  term.tx(13)
    

pub set_clock_regs | check, val

  time.pause(2000)                                              ' allow terminal to open

  term.tx(term#CLS)
  term.str(string("Current clock:"))
  term.str(string(13, "-- "))
  term.str(rtc.time_hhmmss(-1))
  term.str(string(13, "-- ")) 
  term.str(rtc.date_mmddyy(-1))
  term.str(string(13, "-- ")) 
  term.str(rtc.day_long(-1))
  repeat 2
    term.tx(13)

  term.str(string("Would you like to change? [y/N] :"))
  term.rxflush
  check := term.rxtime(3000)                                    ' wait 3s for response
  if ((check <> "y") and (check <> "Y"))
    return
  
  term.tx(term#CLS)
  term.str(string("Set clock registers", 13))

  term.str(string(13, "Hour (0..23)...... "))                   ' show register
  val := get_dec2(0, 23)                                        ' get value
  rtc.wr_reg(rtc#R_HRS, rtc.dec2bcd(val))                       ' set register in rtc

  term.str(string(13, "Minutes (0..59)... "))  
  val := get_dec2(0, 59)                            
  rtc.wr_reg(rtc#R_MNS, rtc.dec2bcd(val))
      
  term.str(string(13, "Seconds (0..59)... "))   
  val := get_dec2(0, 59)                               
  rtc.wr_reg(rtc#R_SCS, rtc.dec2bcd(val))   

  term.str(string(13, "Year (0..99)...... "))   
  val := get_dec2(0, 99)                               
  rtc.wr_reg(rtc#R_YR, rtc.dec2bcd(val))   

  term.str(string(13, "Month (1..12)..... "))   
  val := get_dec2(1, 12)                               
  rtc.wr_reg(rtc#R_MON, rtc.dec2bcd(val))   

  term.str(string(13, "Date (1..31)...... "))   
  val := get_dec2(1, 31)                               
  rtc.wr_reg(rtc#R_DATE, rtc.dec2bcd(val))   

  term.str(string(13, "Day (1..7)........ "))   
  val := get_dec2(1, 7)                               
  rtc.wr_reg(rtc#R_DAY, rtc.dec2bcd(val))

  rtc.wr_reg(rtc#R_CTRL,   $00)   

  
pub get_dec2(lo, hi) | value, len, c

  value := len := 0
  term.rxflush

  repeat
    c := term.rx
    case c
      "0".."9" :                                                ' digit           
        if (len < 2)                                                              
          value := (value * 10) + (c - "0")                                                         
          len += 1                                                                
          term.tx(c)                                                                  

      8 :                                                       ' backspace
        if (len > 0)
          value /= 10
          len -= 1
          term.tx(8)
          term.tx(" ")
          term.tx(8)
        
      13 :
        if ((value => lo) and (value =< hi))
          return value
   
              
dat { license }                                                  

{{

  Copyright (C) 2016 Jon McPhalen

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}